@@ -159,44 +159,57 @@ class SFREAD:
 
     def open(self, *args, **kwargs):
 
-        n_args = len(args) 
-        if n_args == 0:
-            print('No argument given, need at least diag_name')
-            return
-        if isinstance(args[0], str) and len(args[0].strip()) == 3:
-            diag = args[0].strip()
-            if n_args > 1: 
-                if isinstance(args[1], (int, np.integer)):
-                    nshot = args[1]
-        elif isinstance(args[0], (int, np.integer)):
-            nshot = args[0]
-            if n_args > 1:
-                if isinstance(args[1], str) and len(args[1].strip()) == 3:
-                    diag = args[1].strip()
-        if 'nshot' not in locals(): 
-            print('No argument is a shot number (int), taking last AUG shot')
-            nshot = getlastshot()
-        if 'diag' not in locals():
-            diag = input('Please enter a diag_name (str(3), no delimiter):\n')
+        if 'sfh' in kwargs.keys():
+            self.sfile = kwargs['sfh']
+            self.shot = 0
+            self.ed = 0
+            self.diag = os.path.basename(self.sfile)[:3]
 
-        exp = parse_kwargs.parse_kw( ('exp', 'experiment'), kwargs, default='AUGD')
-        ed  = parse_kwargs.parse_kw( ('ed', 'edition'), kwargs, default=0)
+        else:
+
+            n_args = len(args) 
+            if n_args == 0:
+                print('No argument given, need at least diag_name')
+                return
+            if isinstance(args[0], str) and len(args[0].strip()) == 3:
+                diag = args[0].strip()
+                if n_args > 1: 
+                    if isinstance(args[1], (int, np.integer)):
+                        nshot = args[1]
+            elif isinstance(args[0], (int, np.integer)):
+                nshot = args[0]
+                if n_args > 1:
+                    if isinstance(args[1], str) and len(args[1].strip()) == 3:
+                        diag = args[1].strip()
+            if 'nshot' not in locals(): 
+                print('No argument is a shot number (int), taking last AUG shot')
+                nshot = getlastshot()
+            if 'diag' not in locals():
+                diag = input('Please enter a diag_name (str(3), no delimiter):\n')
+
+            exp = parse_kwargs.parse_kw( ('exp', 'experiment'), kwargs, default='AUGD')
+            ed  = parse_kwargs.parse_kw( ('ed', 'edition'), kwargs, default=0)
+
+            self.sfile, self.ed = manage_ed.sf_path(nshot, diag, exp=exp, ed=ed)
+            if self.sfile is None:
+                return
+            else:
+                self.shot = nshot
+                self.diag = diag
+                self.exp  = exp  # unused herein, but useful docu
+
+        if os.path.isfile(self.sfile):
+            self.time = datetime.datetime.fromtimestamp(os.path.getctime(self.sfile))
+            with open(self.sfile, 'rb') as f:
+                byt_str = f.read(128000)
+                if len(byt_str) < 128:
+                    print('Error: shotfile %s has < 128 bytes, ignored' %(self.sfile))
+                    return
+                self.sfh = sfhread.read_sfh(byt_str)
+                self.status = True
+        else:
+            print('Shotfile %s not found' %self.sfile)
 
-        self.sfile, self.ed = manage_ed.sf_path(nshot, diag, exp=exp, ed=ed)
-        if self.sfile is not None:
-            self.shot = nshot
-            self.diag = diag
-            self.exp  = exp  # unused herein, but useful docu
-            if os.path.isfile(self.sfile):
-                self.time = datetime.datetime.fromtimestamp(os.path.getctime(self.sfile))
-                with open(self.sfile, 'rb') as f:
-                    byt_str = f.read(128000)
-                    if len(byt_str) < 128:
-                        print('Error: shotfile %s has < 128 bytes, ignored' %(self.sfile))
-                        self.status = False
-                        return
-                    self.sfh = sfhread.read_sfh(byt_str)
-        self.status = hasattr(self, 'sfh')
         if self.status:
             self.objects = [ lbl for lbl in self.sfh['obj_nam'] if self.sfh[lbl].obj_type in [6, 7, 8, 13] ]
             self.parsets = [ lbl for lbl in self.sfh['obj_nam'] if self.sfh[lbl].obj_type in [3, 4] ]
@@ -415,10 +428,14 @@ class SFREAD:
         buf = self.getchunk(sfo.address, sfo.length)
 
         j0 = 0
-        sfo.par_d = {}
+        par_d = {}
+        
         for j in range(sfo.items):
             pname = str_byt.to_str(buf[j0: j0+8])
             unit, dfmt, n_items = unpack('>3H', buf[j0+8:  j0+14])
+            meta = sfobj.EMPTY()
+#            meta.phys_unit = sfdics.unit_d[unit]
+            meta.data_format = dfmt
             if verbose:
                 print(pname, unit, dfmt, n_items)
             status = unpack('>h', buf[j0+14:  j0+16])[0]
@@ -426,7 +443,7 @@ class SFREAD:
             if dfmt in sfdics.fmt2len.keys(): # char variable
                 dlen = sfdics.fmt2len[dfmt]
                 bytlen = n_items * dlen
-                value = np.chararray((n_items,), itemsize=dlen, buffer=buf[j0+2: j0+2+bytlen])
+                data = np.chararray((n_items,), itemsize=dlen, buffer=buf[j0+2: j0+2+bytlen])
                 dj0 = 8 * ( (bytlen + 9)//8 )
                 j0 += dj0
             else: # number
@@ -435,18 +452,18 @@ class SFREAD:
                 bytlen = val_len * np.dtype(sfmt).itemsize
                 fmt = '>%d%s' %(val_len, sfmt)
                 if dfmt == 7:
-                    value = unpack(sfmt, buf[j0+5: j0+6])[0]
+                    data = unpack(sfmt, buf[j0+5: j0+6])[0]
                 else:
-                    value = unpack(fmt, buf[j0: j0+bytlen])
-                    value = np.squeeze(value[2:])
+                    data = unpack(fmt, buf[j0: j0+bytlen])
+                    data = np.squeeze(data[2:])
                 dj0 = 8 * ( (bytlen + 7)//8 )
                 j0 += dj0
 
             if verbose:
-                print(pname, value)
-            sfo.par_d[pname] = value
+                print(pname, data)
+            par_d[pname] = sfobj.SFOBJ(data, sfho=meta)
 
-        return sfo.par_d
+        return par_d
 
 
     def getlist(self, obj=None):
